// ************************************************************************** //
//
//  BornAgain: simulate and fit scattering at grazing incidence
//
//! @file      coregui/mainwindow/mainwindow.h
//! @brief     Declares class MainWindow
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2016
//! @authors   Scientific Computing Group at MLZ Garching
//! @authors   Céline Durniak, Marina Ganeva, David Li, Gennady Pospelov
//! @authors   Walter Van Herck, Joachim Wuttke
//
// ************************************************************************** //

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "WinDllMacros.h"
#include "fancymainwindow.h"
#include <QMainWindow>

namespace Manhattan {
    class FancyTabWidget;
    class ProgressBar;
}

class WelcomeView;
class InstrumentView;
class SampleView;
class SimulationView;
class JobView;
class SessionModelView;
class ObsoleteFitView;

class MaterialModel;
class MaterialSvc;
class SampleModel;
class InstrumentModel;
class JobModel;
class ObsoleteFitModel;
class ApplicationModels;
class ProjectManager;
class ActionManager;
class QSettings;
class ToolTipDataBase;
class UpdateNotifier;


class BA_CORE_API_ MainWindow : public Manhattan::FancyMainWindow
{
    Q_OBJECT

public:
    enum ETabViewId {WELCOME, INSTRUMENT, SAMPLE, SIMULATION, JOB, TESTVIEW, MAXVIEWCOUNT};

    explicit MainWindow(QWidget *parent = 0);

    MaterialModel *materialModel();
    InstrumentModel *instrumentModel();
    SampleModel *sampleModel();
    JobModel *jobModel();
    ObsoleteFitModel *fitModel();
    ApplicationModels *models();

    Manhattan::ProgressBar *progressBar();
    QStatusBar *statusBar();

    ActionManager *getActionManager();
    ProjectManager *projectManager();
    UpdateNotifier *getUpdateNotifier();

public slots:
    void onChangeTabWidget(int index);
    void onFocusRequest(int index);
    void openRecentProject();
    void onRunSimulationShortcut();
    void onAboutApplication();
    void onSessionModelViewActive(bool isActive);

protected:
    virtual void closeEvent(QCloseEvent *event);
    virtual void showEvent(QShowEvent *event);

private:
    void initApplication();
    void initProgressBar();
    void initViews();
    void readSettings();
    void writeSettings();
    void initConnections();

    Manhattan::FancyTabWidget  *m_tabWidget;
    Manhattan::ProgressBar *m_progressBar;

    ApplicationModels *m_applicationModels;
    ProjectManager *m_projectManager;
    ActionManager *m_actionManager;
    ToolTipDataBase *m_toolTipDataBase;
    UpdateNotifier *m_updateNotifier;

    WelcomeView *m_welcomeView;
    InstrumentView *m_instrumentView;
    SampleView *m_sampleView;
    SimulationView *m_simulationView;
    JobView *m_jobView;
    SessionModelView *m_sessionModelView;
    ObsoleteFitView *m_fitView;
};

#endif // MAINWINDOW_H

